package com.tzld.piaoquan.recommend.server.implement;


import com.tzld.piaoquan.recommend.server.common.base.RankItem;
import com.tzld.piaoquan.recommend.server.framework.candidiate.Candidate;
import com.tzld.piaoquan.recommend.server.framework.common.User;
import com.tzld.piaoquan.recommend.server.framework.merger.MergeUtils;
import com.tzld.piaoquan.recommend.server.framework.merger.SimilarityUtils;
import com.tzld.piaoquan.recommend.server.framework.merger.StrategyQueue;
import com.tzld.piaoquan.recommend.server.framework.recaller.BaseRecaller;
import com.tzld.piaoquan.recommend.server.framework.recaller.provider.RedisBackedQueue;
import com.tzld.piaoquan.recommend.server.framework.utils.RedisSmartClient;
import com.tzld.piaoquan.recommend.server.gen.recommend.RecommendRequest;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class TopRecommendPipeline {

    public static final String FILTER_CONF = "filter_config.conf";
    public static final String MERGE_CONF = "merge_config.conf";

    public static final String PREFIX = "";

    @Resource
    private RedisSmartClient client;

    public List<RankItem> feedByRec(final RecommendRequest requestData,
                                    final int requestIndex,
                                    final User userInfo) {
        int recallNum =  200;

        // Step 1: Attention extraction
//        long timestamp = System.currentTimeMillis();
//        UserAttentionExtractorPipeline attentionExtractorPipeline = UserAttentionExtractorUtils.getAtttentionPipeline(UserAttentionExtractorUtils.BASE_CONF);
//        attentionExtractorPipeline.extractAttention(requestData, userInfo);

        // Step 2: create top queue
        StrategyQueue topQueue = MergeUtils.createTopQueue(MERGE_CONF, "top-queue");


        // Step 3: Candidate
        Map<String, Candidate> candidates = new HashMap<String, Candidate>();
        topQueue.candidate(candidates, recallNum, userInfo, requestData, 0, 0);


        // Step 4: Recalling & Basic Scoring
        RedisBackedQueue queueProvider = new RedisBackedQueue(client, 1000L);

        BaseRecaller recaller = new BaseRecaller(queueProvider);
        List<RankItem> items = recaller.recalling(requestData, userInfo, requestIndex, new ArrayList<>(candidates.values()));


        // Step 4: Advance Scoring
//        timestamp = System.currentTimeMillis();
//        ScorerPipeline scorerPipeline = getScorerPipeline(requestData);
//        items = scorerPipeline.scoring(requestData, userInfo, requestIndex, items);


        // Step 5: Merger
        MergeUtils.distributeItemsToMultiQueues(topQueue, items);
        topQueue.merge(recallNum * 3, userInfo, requestData, requestIndex, 0);

        // step 6：多样性融合
        List<RankItem> mergeItems = topQueue.getItems();
        MergeUtils.diversityRerank(mergeItems, SimilarityUtils.getIsSameUserTagOrCategoryFunc(), recallNum, 6, 2);


        //
        // timestamp = System.currentTimeMillis();
        // RankPipeline rankPipeline = getRankPipeline(requestData;
        // List<RankItem> resultItems = rankPipeline.doRank(requestData, userInfo, requestIndex, items);


        // Step 6: Global Rank & subList
        // TODO: Global Rank


        return items;
    }


}
